love.window.setMode(1200, 600, {vsync = false, resizable = true, minwidth = 600, minheight = 400})
love.graphics.setDefaultFilter("nearest", "nearest")
love.window.setTitle("Pixel")
love.graphics.setBackgroundColor(120, 120, 120)
love.keyboard.setKeyRepeat(true)
love.graphics.setPointStyle("rough")

require("pixel")

if not love.filesystem.exists("images") then
	love.filesystem.createDirectory("images")
end

local font_small = love.graphics.newFont(16)
local font_big = love.graphics.newFont(24)
local font_loading = love.graphics.newFont(48)

function refreshFiles()
	files = { }
	for d,s in ipairs(love.filesystem.getDirectoryItems("images")) do
		if love.filesystem.isFile("images/" .. s) then
			files[#files+1] = s
		end
	end
	selected = 0
	scroll = 0
end
refreshFiles()

screen = {w = 1200, h = 600}
mouse = {x = 0, y = 0}

--settings
resolution = 128
resolution_final = 128
resolution_max = 1024
shadowStrength = 8
bloomStrength = 8
pixelPower = 6

bars = 6

love.graphics.rectangle_old = love.graphics.rectangle
love.graphics.rectangle = function(mode, x, y, w, h)
	love.graphics.rectangle_old(mode, x - 0.5, y - 0.5, w, h)
end

function hover(x, y, w, h)
	if mouse.x > x and mouse.x < x+w and mouse.y > y and mouse.y < y+h then
		return true
	else
		return false
	end
end

function love.draw()
	love.graphics.setColor(0, 0, 0)
	love.graphics.rectangle("line", 1, 1, 200, 30)
	love.graphics.rectangle("line", 1, 30, 200, screen.h - 210)
	
	love.graphics.rectangle("line", 51, screen.h - 60, 200-50, 30)
	love.graphics.rectangle("line", 51, screen.h - 30, 200-50, 30)
	love.graphics.rectangle("line", 1, screen.h - 60, 50, 60)
	
	love.graphics.setFont(font_small)
	love.graphics.print(love.timer.getFPS() .. " FPS", 5, 10)
	
	for i = 1 + scroll, math.floor((screen.h-90-30*bars)/30) + scroll do
		if files[i] then
			if i == selected then
				love.graphics.setColor(90, 90, 90)
				love.graphics.rectangle("fill", 1, (i-1) * 30 + 30, 200, 30)
			end
			local ii = i-scroll
			if hover(1, (ii-1) * 30 + 30, 200, 30) then
				love.graphics.setColor(140, 140, 140)
				love.graphics.rectangle("fill", 1, (ii-1) * 30 + 30, 200, 30)
				if love.mouse.isDown("l") and selected ~= ii then
					selected = i
				end
			end
			love.graphics.setColor(0, 0, 0)
			love.graphics.print(files[i], 5, (ii-1) * 30 + 30 + 8)
			love.graphics.rectangle("line", 1, (ii-1) * 30 + 30, 200, 30)
		else
			break
		end
	end
	
	love.graphics.setColor(140, 140, 140)
	if hover(1, screen.h-60, 50, 60) then
		love.graphics.rectangle("fill", 2, screen.h-60+1, 50-2, 60-2)
	end
	if hover(50, screen.h-60, 150, 30) then
		love.graphics.rectangle("fill", 52, screen.h-60+1, 150-2, 30-2)
	end
	if hover(50, screen.h-30, 150, 30) then
		love.graphics.rectangle("fill", 52, screen.h-30+1, 150-2, 30-2)
	end
	
	--bars
	love.graphics.rectangle("fill", 2, screen.h-240, resolution_final/1024*200-2, 29)
	love.graphics.rectangle("fill", 2, screen.h-210, resolution/1024*200-2, 29)
	love.graphics.rectangle("fill", 2, screen.h-180, resolution_max/love.graphics.getSystemLimit("texturesize")*200-2, 29)
	love.graphics.rectangle("fill", 2, screen.h-150, shadowStrength/16*200-2, 29)
	love.graphics.rectangle("fill", 2, screen.h-120, bloomStrength/16*200-2, 29)
	love.graphics.rectangle("fill", 2, screen.h-90, pixelPower/40*200-2, 29)
	love.graphics.setColor(140, 140, 140)
	
	love.graphics.setColor(0, 0, 0)
	love.graphics.print(resolution_final .. "px resolution", 10, screen.h-240+5)
	love.graphics.print(resolution .. "px rendersize", 10, screen.h-210+5)
	love.graphics.print(resolution_max .. "px max res", 10, screen.h-180+5)
	love.graphics.print(shadowStrength .. " shadow effect", 10, screen.h-150+5)
	love.graphics.print(bloomStrength .. " bloom", 10, screen.h-120+5)
	love.graphics.print(pixelPower .. " pixel strenght", 10, screen.h-90+5)
	
	love.graphics.rectangle("fill", 1, screen.h - 60-30*bars-2, 200, 2)
	for i = 1, bars do
		love.graphics.rectangle("line", 1, screen.h - 60-30*i, 200, 30)
	end
	
	love.graphics.printf("open\ndir", 0, screen.h - 50, 50, "center")
	love.graphics.setFont(font_big)
	love.graphics.printf("PIXEL!", 50, screen.h - 60, 150, "center")
	love.graphics.printf("PIXEL ALL!", 50, screen.h - 30, 150, "center")
	
	--draw images
	if canvas then
		love.graphics.setColor(255, 255, 255)
		local scale = math.min((screen.w-200-20) / canvas:getWidth(), (screen.h-20) / canvas:getHeight())
		love.graphics.draw(canvas, 200 + 10, 10, 0, scale)
	end
end

function love.update(dt)
	mouse.x, mouse.y = love.mouse.getPosition()
	
	love.timer.sleep(1/60)
	
	if love.mouse.isDown("l") then
		if mouse.x < 200 and mouse.y > screen.h-240 and mouse.y < screen.h-210 then
			resolution = math.floor(mouse.x / 200 * 1024 / 32 + 1) * 32
			resolution_final = math.floor(mouse.x / 200 * 1024 / 32 + 1) * 32
		end
		if mouse.x < 200 and mouse.y > screen.h-210 and mouse.y < screen.h-180 then
			resolution = math.floor(mouse.x / 200 * 1024 / 32 + 1) * 32
		end
		if mouse.x < 200 and mouse.y > screen.h-180 and mouse.y < screen.h-150 then
			resolution_max = math.floor(mouse.x / 200 * love.graphics.getSystemLimit("texturesize") / 256 + 1) * 256
		end
		if mouse.x < 200 and mouse.y > screen.h-150 and mouse.y < screen.h-120 then
			shadowStrength = math.floor(mouse.x / 200 * 16 + 0.8)
		end
		if mouse.x < 200 and mouse.y > screen.h-120 and mouse.y < screen.h-90 then
			bloomStrength = math.floor(mouse.x / 200 * 16 + 0.8)
		end
		if mouse.x < 200 and mouse.y > screen.h-90 and mouse.y < screen.h-60 then
			pixelPower = math.floor(mouse.x / 200 * 40)+1
		end
	end
end

function love.keypressed(key)
	if key == "down" then
		scroll = math.min(math.max(0, #files - math.floor((screen.h-90-30*bars)/30)), scroll + 1)
	elseif key == "up" then
		scroll = math.max(0, scroll - 1)
	end
	
	if key == "f5" then
		refreshFiles()
	end
end

function love.mousepressed(x, y, button)
	if button == "wd" then
		scroll = math.min(math.max(0, #files - math.floor((screen.h-90-30*bars)/30)), scroll + 1)
	elseif button == "wu" then
		scroll = math.max(1, scroll - 1)
	end
end

function love.mousereleased(x, y, button)
	if button == "l" then
		if hover(1, screen.h-60, 50, 60) then
			love.system.openURL(love.filesystem.getSaveDirectory() .. "/images")
		end
		
		if files[selected] then
			if hover(50, screen.h-60, 150, 30) then
				canvas = pixelImage("images/" .. files[selected] or "noImageSelected", "output.png")
			end
		end
		
		if hover(50, screen.h-30, 150, 30) then
			love.filesystem.createDirectory("output")
			
			local count = 0
			local imgCount = 0
			
			--count images
			local function recursive(path)
				for d,s in ipairs(love.filesystem.getDirectoryItems("images/" .. path)) do
					if love.filesystem.isDirectory("images/" .. path .. "/" .. s) then
						love.filesystem.createDirectory("output/" .. path .. "/" .. s)
						recursive(path .. "/" .. s)
					elseif s:sub(#s-3) == ".png" then
						imgCount = imgCount + 1
					end
				end
			end
			recursive("")
			
			--render everything
			local function recursive(path)
				for d,s in ipairs(love.filesystem.getDirectoryItems("images/" .. path)) do
					if love.filesystem.isDirectory("images/" .. path .. "/" .. s) then
						love.filesystem.createDirectory("output/" .. path .. "/" .. s)
						recursive(path .. "/" .. s)
					elseif s:sub(#s-3) == ".png" then
						love.graphics.setColor(120, 120, 120)
						love.graphics.rectangle("fill", 200+5, 5, screen.w-200-10, screen.h-10)
						love.graphics.setColor(0, 0, 0)
						love.graphics.setFont(font_loading)
						love.graphics.printf(count .. " of " .. imgCount .. "\n" .. math.floor(count/imgCount*100) .. "%", 200, screen.h/2-100, screen.w-200, "center")
						love.graphics.present()
						pixelImage("images/" .. path .. "/" .. s, "output/" .. path .. "/" .. s)
						count = count + 1
					end
				end
			end
			recursive("")
		end
	end
end

function love.resize(w, h)
	screen.w = w
	screen.h = h
end