function loadImage(path)
	if love.filesystem.exists(path) then
		return love.graphics.newImage(path)
	end
end

pixelShader = love.graphics.newShader[[
	extern float pixelPower;
	extern vec2 res;
	extern int shadowStrength;
	float rand(vec2 co) {
		return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
	}
	float simplePow(float x) {
		return x*x;
	}
	
	vec4 effect(vec4 color, Image texture, vec2 tc, vec2 screen_coords) {
		vec4 col = Texel(texture, tc);
		float shadow = 0;
		float r = 0;
		float g = 0;
		float b = 0;
		
		//pixel
		float e = rand(vec2(tc)) * pixelPower;
		r = max(0.0, min(1.0, col.r+e));
		g = max(0.0, min(1.0, col.g+e));
		b = max(0.0, min(1.0, col.b+e));
		
		//contur
		float b1 = (col.r+col.g+col.b)/3;
		float b2 = 0;
		for (int x = -1; x <= 1; x++) {
			for (int y = -1; y <= 1; y++) {
				vec4 col2 = Texel(texture, tc + (x/(res[0]/8+96), y/(res[1]/8+96)));
				b2 = b2 + (col2.r+col2.g+col2.b)/3/9;
			}
		}
		float d = sqrt(simplePow(sqrt(b1*b1) - sqrt(b2*b2)));
		int a = 6;
		d = pow(min(1, max(0, d*a - simplePow(d*a) / 4)), 0.8) / 2 / 5;
		shadow = -d;
		
		//shadow
		int size = int(max(res[0], res[1])/256)+1;
		b1 = (col.r+col.g+col.b)/3;
		b2 = 0;
		for (int x = -size; x <= size; x++) {
			for (int y = -size; y <= size; y++) {
				vec4 col2 = Texel(texture, tc + (x/(res[0]/8+96), y/(res[1]/8+96)));
				b2 = b2 + (col2.r+col2.g+col2.b)/3/simplePow(size*2+1);
			}
		}
		if (b1-b2 > 0) {
			shadow = shadow + min(1, (b1-b2)*shadowStrength/8);
		}
		
		//calculate color
		r = min(1, max(0, r + shadow));
		g = min(1, max(0, g + shadow));
		b = min(1, max(0, b + shadow));
		
		return vec4(r, g, b, col.a);
	}
]]

function pixelImage(path_origin, path)
	local image = loadImage(path_origin)
	
	--final texture cant be larger than rendered texture
	resolution_final = math.min(resolution, resolution_final)
	
	if image then
		--calculate best height and width for your new image
		local width = image:getWidth() / math.min(image:getWidth(), image:getHeight()) * resolution
		local height = image:getHeight() / math.min(image:getWidth(), image:getHeight()) * resolution
		
		--anti-too-large-protection
		while true do
			local f = math.max(width/resolution_max, height/resolution_max)
			if f > 1 then
				width = width/2
				height = height/2
			else
				break
			end
		end
		
		local canvas_pre = love.graphics.newCanvas(width, height)
		local canvas_pre_final = love.graphics.newCanvas(width, height)
		local canvas = love.graphics.newCanvas(width*(resolution_final/resolution), height*(resolution_final/resolution))
		
		--sends some important vars
		pixelShader:send("pixelPower", pixelPower/255)
		pixelShader:sendInt("shadowStrength", shadowStrength)
		pixelShader:send("res", {width, height})
		love.graphics.setColor(255, 255, 255)
		
		--scale image
		love.graphics.setCanvas(canvas_pre)
		love.graphics.draw(image, 0, 0, 0, width/image:getWidth(), height/image:getHeight())
		love.graphics.setCanvas()
		
		--render image
		love.graphics.setCanvas(canvas_pre_final)
		love.graphics.setShader(pixelShader)
		love.graphics.draw(canvas_pre)
		love.graphics.setShader()
		
		love.graphics.setCanvas(canvas)
		love.graphics.draw(canvas_pre_final, 0, 0, 0, resolution_final/resolution)
		love.graphics.setCanvas()
		
		--save image
		canvas:getImageData():encode(path)
		return canvas
	end
end